@@ -1,5 +1,5 @@
 ### Terrence D. Jorgensen & Yves Rosseel
-### Last updated: 1 September 2018
+### Last updated: 29 August 2019
 ### Pooled likelihood ratio test for multiple imputations
 ### Borrowed source code from lavaan/R/lav_test_LRT.R
 
@@ -32,7 +32,6 @@
 ##' @aliases lavTestLRT.mi
 ##' @importFrom lavaan lavListInspect parTable lavTestLRT
 ##' @importFrom stats cov pchisq pf
-##' @importFrom methods getMethod
 ##'
 ##' @param object,h1 An object of class \code{\linkS4class{lavaan.mi}}.
 ##'   \code{object} should be nested within (more constrained than) \code{h1}.
@@ -43,6 +42,15 @@
 ##'   calculated using each imputed data set, which will then be pooled across
 ##'   imputations, as described in Li, Meng, Raghunathan, & Rubin (1991).
 ##'   Find additional details in Enders (2010, chapter 8).
+##' @param omit.imps \code{character} vector specifying criteria for omitting
+##'   imputations from pooled results.  Can include any of
+##'   \code{c("no.conv", "no.se", "no.npd")}, the first 2 of which are the
+##'   default setting, which excludes any imputations that did not
+##'   converge or for which standard errors could not be computed.  The
+##'   last option (\code{"no.npd"}) would exclude any imputations which
+##'   yielded a nonpositive definite covariance matrix for observed or
+##'   latent variables, which would include any "improper solutions" such
+##'   as Heywood cases.
 ##' @param asymptotic \code{logical}. If \code{FALSE} (default), the pooled test
 ##'   will be returned as an \emph{F}-distributed statistic with numerator
 ##'   (\code{df1}) and denominator (\code{df2}) degrees of freedom.
@@ -61,11 +69,13 @@
 ##'
 ##' @return
 ##'   A vector containing the LRT statistic (either an \code{F} or \eqn{\chi^2}
-##'   statistic, depending on the \code{asymptotic} argument), the degrees of
-##'   freedom (numerator and denominator, if \code{asymptotic = FALSE}), and a
-##'   \emph{p} value. Robust statistics will also include the average (across)
-##'   imputations) scaling factor and (if relevant) shift parameter(s), unless
-##'   \code{pool.robust = TRUE}.
+##'   statistic, depending on the \code{asymptotic} argument), its degrees of
+##'   freedom (numerator and denominator, if \code{asymptotic = FALSE}), its
+##'   \emph{p} value, and 2 missing-data diagnostics: the relative invrease
+##'   in variance (RIV, or average for multiparameter tests: ARIV) and the
+##'   fraction missing information (FMI = ARIV / (1 + ARIV)). Robust statistics
+##'   will also include the average (across imputations) scaling factor and
+##'   (if relevant) shift parameter(s), unless \code{pool.robust = TRUE}.
 ##'
 ##' @author
 ##'   Terrence D. Jorgensen (University of Amsterdam;
@@ -78,11 +88,11 @@
 ##'   Li, K.-H., Meng, X.-L., Raghunathan, T. E., & Rubin, D. B. (1991).
 ##'   Significance levels from repeated \emph{p}-values with multiply-imputed
 ##'   data. \emph{Statistica Sinica, 1}(1), 65--92. Retrieved from
-##'   \url{https://www.jstor.org/stable/24303994}
+##'   https://www.jstor.org/stable/24303994
 ##'
 ##'   Meng, X.-L., & Rubin, D. B. (1992). Performing likelihood ratio tests with
-##'   multiply-imputed data sets. \emph{Biometrika, 79}(1), 103--111. Retrieved
-##'   from \url{https://www.jstor.org/stable/2337151}
+##'   multiply-imputed data sets. \emph{Biometrika, 79}(1), 103--111.
+##'   doi:10.2307/2337151
 ##'
 ##'   Rubin, D. B. (1987). \emph{Multiple imputation for nonresponse in surveys}.
 ##'   New York, NY: Wiley.
@@ -128,27 +138,67 @@
 ##'
 ##' @export
 lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
+                          omit.imps = c("no.conv","no.se"),
                           asymptotic = FALSE, pool.robust = FALSE, ...) {
   ## check class
   if (!inherits(object, "lavaan.mi")) stop("object is not class 'lavaan.mi'")
-  useImps <- sapply(object@convergence, "[[", i = "converged")
-  nImps <- sum(useImps)
-  DF0 <- object@testList[[ which(useImps)[1] ]][[1]][["df"]]
+
+  useImps <- rep(TRUE, length(object@DataList))
+  if ("no.conv" %in% omit.imps) useImps <- sapply(object@convergence, "[[", i = "converged")
+  if ("no.se" %in% omit.imps) useImps <- useImps & sapply(object@convergence, "[[", i = "SE")
+  if ("no.npd" %in% omit.imps) {
+    Heywood.lv <- sapply(object@convergence, "[[", i = "Heywood.lv")
+    Heywood.ov <- sapply(object@convergence, "[[", i = "Heywood.ov")
+    useImps <- useImps & !(Heywood.lv | Heywood.ov)
+  }
+  m <- sum(useImps)
+  if (m == 0L) stop('No imputations meet "omit.imps" criteria.')
+  useImps <- which(useImps)
+
+  DF0 <- object@testList[[ useImps[1] ]]$standard[["df"]]
 
   ## model comparison?
   if (!is.null(h1)) {
     if (!inherits(h1, "lavaan.mi")) stop("h1 is not class 'lavaan.mi'")
-    if (lavListInspect(object, "options")$test != lavListInspect(h1, "options")$test) {
-      stop('Different test statistics were requested for the 2 models.')
+    if (!all(lavListInspect(object, "options")$test == lavListInspect(h1, "options")$test)) {
+      stop('Different (sets of) test statistics were requested for the 2 models.')
     }
-    if (any(useImps != sapply(h1@convergence, "[[", i = "converged")))
+
+    useImps1 <- rep(TRUE, length(h1@DataList))
+    if ("no.conv" %in% omit.imps) useImps1 <- sapply(h1@convergence, "[[", i = "converged")
+    if ("no.se" %in% omit.imps) useImps1 <- useImps1 & sapply(h1@convergence, "[[", i = "SE")
+    if ("no.npd" %in% omit.imps) {
+      Heywood.lv <- sapply(h1@convergence, "[[", i = "Heywood.lv")
+      Heywood.ov <- sapply(h1@convergence, "[[", i = "Heywood.ov")
+      useImps1 <- useImps1 & !(Heywood.lv | Heywood.ov)
+    }
+    m <- sum(useImps1)
+    if (m == 0L) stop('No imputations meet "omit.imps" criteria.')
+    useImps1 <- which(useImps1)
+
+    h0.not.h1 <- setdiff(useImps, useImps1)
+    if (length(h0.not.h1)) warn0 <- paste('\n\nFor the following imputations, ',
+                                          '"object" converged but not "h1":',
+                                          paste(h0.not.h1, collapse = ", "),
+                                          '\n\n')
+    h1.not.h0 <- setdiff(useImps1, useImps)
+    if (length(h1.not.h0)) warn1 <- paste('\n\nFor the following imputations, ',
+                                          '"h1" converged but not "object":',
+                                          paste(h1.not.h0, collapse = ", "),
+                                          '\n\n')
+    if (length(c(h0.not.h1, h1.not.h0)))
       warning('The models being compared did not converge on the same set of ',
-              'imputations. Likelihood ratio test conducted using only the ',
+              'imputations. ',
+              if (length(h0.not.h1)) warn0 else '',
+              if (length(h1.not.h0)) warn1 else '',
+              'Likelihood ratio test conducted using only the ',
               'imputations for which both models converged.')
-    useImps <- useImps & sapply(h1@convergence, "[[", i = "converged")
-    nImps <- sum(useImps)
+    useImps <- intersect(useImps, useImps1)
+    m <- length(useImps) # yes, redefine if necessary
+    if (m == 0L) stop('For no imputations did both models converge.')
+
     ## check DF
-    DF1 <- h1@testList[[ which(useImps)[1] ]][[1]][["df"]]
+    DF1 <- h1@testList[[ useImps[1] ]]$standard[["df"]]
     if (DF0 == DF1) stop("models have equal degrees of freedom")
     if (DF0 < DF1) {
       H0 <- h1
@@ -164,7 +214,7 @@ lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
   ## only keep arguments relevant to pass to lavTestLRT (if D2)
   dots <- list(...)[names(formals(lavTestLRT))]
 
-  ## check test options, backward compatibility?
+  ## check test-pooling options, for backward compatibility?
   test <- tolower(test[1])
   if (test == "mplus") {
     test <- "D3"
@@ -179,11 +229,26 @@ lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
   }
 
   ## check for robust
-  robust <- lavListInspect(object, "options")$test != "standard"
-  #TODO: check for bollen-stine bootstrap test
+  test.names <- lavListInspect(object, "options")$test
+  # lavaan 0.6-5: for now, we only acknowledge the first non-standard @test
+  if (length(test.names) > 1L) {
+    ## remove standard and any bootstrapped tests
+    rm.idx <- which(test.names %in% c("standard","bootstrap","bollen.stine"))
+    if (length(rm.idx) > 0L) {
+      test.names <- test.names[-rm.idx]
+    }
+    ## only acknowledge the first scaled test statistic
+    if (length(test.names) > 1L) {
+      test.names <- test.names[1]
+    }
+  }
+
+  robust <- any(test.names %in% c("satorra.bentler","yuan.bentler",
+                                  "yuan.bentler.mplus","scaled.shifted",
+                                  "mean.var.adjusted","satterthwaite"))
   if (!robust) pool.robust <- FALSE
 
-  scaleshift <- lavListInspect(object, "options")$test == "scaled.shifted"
+  scaleshift <- any(test.names == "scaled.shifted")
   if (scaleshift && !is.null(h1)) {
     if (test == "D3" | !pool.robust)
       message("If test = 'scaled.shifted' (estimator = 'WLSMV' or 'MLMV'), ",
@@ -211,15 +276,17 @@ lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
   if (robust && pool.robust) {
     ## pool both the naive and robust test statistics, return both to
     ## make output consistent across options
-    out.naive <- D2.LRT(object, h1 = h1, asymptotic = TRUE, pool.robust = FALSE)
-    out.robust <- D2.LRT(object, h1 = h1, asymptotic = TRUE,
-                         pool.robust = TRUE, LRTargs = dots)
+    out.naive <- D2.LRT(object, h1 = h1, useImps = useImps,
+                        asymptotic = asymptotic, pool.robust = FALSE)
+    out.robust <- D2.LRT(object, h1 = h1, useImps = useImps, LRTargs = dots,
+                         asymptotic = asymptotic, pool.robust = TRUE)
     out <- c(out.naive, out.robust)
   } else if (test == "D2") {
-    out <- D2.LRT(object, h1 = h1, asymptotic = asymptotic,
+    out <- D2.LRT(object, h1 = h1, useImps = useImps, asymptotic = asymptotic,
                   pool.robust = pool.robust)
   } else if (test == "D3") {
-    out <- D3.LRT(object, h1 = h1, asymptotic = asymptotic)
+    out <- D3.LRT(object, h1 = h1, useImps = useImps, asymptotic = asymptotic,
+                  omit.imps = omit.imps)
   }
 
   ## If test statistic is negative, return without any indices or robustness
@@ -243,14 +310,14 @@ lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
 
   ## If robust statistics were not pooled above, robustify naive statistics
   if (robust & !pool.robust) {
-    out <- robustify(ChiSq = out, object, h1, useImps)
+    out <- robustify(ChiSq = out, object = object, h1 = h1, useImps = useImps)
     if (scaleshift) {
       extraWarn <- ' and shift parameter'
-    } else if (lavListInspect(object, "options")$test == "mean.var.adjusted") {
+    } else if (any(test.names == "mean.var.adjusted")) {
       extraWarn <- ' and degrees of freedom'
     } else extraWarn <- ''
     message('Robust corrections are made by pooling the naive chi-squared ',
-            'statistic across ', nImps, ' imputations for which the model ',
+            'statistic across ', m, ' imputations for which the model ',
             'converged, then applying the average (across imputations) scaling',
             ' factor', extraWarn, ' to that pooled value. \n',
             'To instead pool the robust test statistics, set test = "D2" and ',
@@ -269,17 +336,8 @@ lavTestLRT.mi <- function(object, h1 = NULL, test = c("D3","D2"),
 
 
 ##' @importFrom lavaan lavListInspect parTable lavTestLRT
-D2.LRT <- function(object, h1 = NULL, asymptotic = FALSE,
+D2.LRT <- function(object, h1 = NULL, useImps, asymptotic = FALSE,
                    pool.robust = FALSE, LRTargs = list()) {
-  useImps <- sapply(object@convergence, "[[", i = "converged")
-  if (!is.null(h1)) {
-    if (any(useImps != sapply(h1@convergence, "[[", i = "converged")))
-      warning('The models being compared did not converge on the same set of ',
-              'imputations. Likelihood ratio test conducted using only the ',
-              'imputations for which both models converged.')
-    useImps <- useImps & sapply(h1@convergence, "[[", i = "converged")
-  }
-
   warn <- lavListInspect(object, "options")$warn
 
   if (pool.robust && !is.null(h1)) {
@@ -316,8 +374,10 @@ D2.LRT <- function(object, h1 = NULL, asymptotic = FALSE,
     noLRT <- sapply(FIT@funList, function(x) x[1] == "lavTestLRT() failed")
     if (all(noFit | noLRT)) stop("No success using lavTestScore() on any imputations.")
 
-    chiList <- sapply(FIT@funList[useImps & !(noFit | noLRT)], "[[", i = "chisq")
-    dfList <- sapply(FIT@funList[useImps & !(noFit | noLRT)], "[[", i = "df")
+    chiList <- sapply(FIT@funList[ intersect(which(!(noFit | noLRT)), useImps) ],
+                      "[[", i = "chisq")
+    dfList <- sapply(FIT@funList[ intersect(which(!(noFit | noLRT)), useImps) ],
+                     "[[", i = "df")
     out <- calculate.D2(chiList, DF = mean(dfList), asymptotic)
     names(out) <- paste0(names(out), ".scaled")
     class(out) <- c("lavaan.vector","numeric")
@@ -326,18 +386,32 @@ D2.LRT <- function(object, h1 = NULL, asymptotic = FALSE,
   ## else, return model fit OR naive difference test to be robustified
 
 
-  test <- if (pool.robust) 2L else 1L
+  test.names <- lavListInspect(object, "options")$test
+  # lavaan 0.6-5: for now, we only acknowledge the first non-standard @test
+  if (length(test.names) > 1L) {
+    ## remove standard and any bootstrapped tests
+    rm.idx <- which(test.names %in% c("standard","bootstrap","bollen.stine"))
+    if (length(rm.idx) > 0L) {
+      test.names <- test.names[-rm.idx]
+    }
+    ## only acknowledge the first scaled test statistic
+    if (length(test.names) > 1L) {
+      test.names <- test.names[1]
+    }
+  }
+
   ## pool Wald tests
   if (is.null(h1)) {
-    DF <- mean(sapply(object@testList[useImps], function(x) x[[test]][["df"]]))
-    w <- sapply(object@testList[useImps], function(x) x[[test]][["stat"]])
+    test <- if (pool.robust) test.names[1] else "standard"
+    DF <- mean(sapply(object@testList[useImps], function(x) x[[test]][["df"]]  ))
+    w  <-      sapply(object@testList[useImps], function(x) x[[test]][["stat"]])
   } else {
     ## this will not get run if !pool.robust because logic catches that first
-    DF0 <- mean(sapply(object@testList[useImps], function(x) x[[1]][["df"]]))
-    DF1 <- mean(sapply(h1@testList[useImps], function(x) x[[1]][["df"]]))
+    DF0 <- mean(sapply(object@testList[useImps], function(x) x$standard[["df"]]))
+    DF1 <- mean(sapply(    h1@testList[useImps], function(x) x$standard[["df"]]))
     DF <- DF0 - DF1
-    w0 <- sapply(object@testList[useImps], function(x) x[[1]][["stat"]])
-    w1 <- sapply(h1@testList[useImps], function(x) x[[1]][["stat"]])
+    w0 <- sapply(object@testList[useImps], function(x) x$standard[["stat"]])
+    w1 <- sapply(    h1@testList[useImps], function(x) x$standard[["stat"]])
     w <- w0 - w1
     ## check DF
     if (DF < 0) {
@@ -361,7 +435,8 @@ D2.LRT <- function(object, h1 = NULL, asymptotic = FALSE,
 
 ##' @importFrom lavaan parTable lavaan lavListInspect
 ##' @importFrom methods getMethod
-getLLs <- function(object, useImps, saturated = FALSE) {
+getLLs <- function(object, useImps, saturated = FALSE,
+                   omit.imps = c("no.conv","no.se")) {
   ## FIXME: lavaanList does not return info when fixed because no convergence!
   dataList <- object@DataList[useImps]
   lavoptions <- lavListInspect(object, "options")
@@ -372,20 +447,26 @@ getLLs <- function(object, useImps, saturated = FALSE) {
   if (length(cluster) == 0L) cluster <- NULL
 
   if (saturated) {
-    fit <- lavaan(parTable(object), data = dataList[[ which(useImps)[1] ]],
-                  slotOptions = lavoptions, group = group, cluster = cluster)
-    ## use saturated parameter table as new model
-    PT <- lavaan::lav_partable_unrestricted(fit)
-    ## fit saturated parameter table to each imputation, return estimates
-    satParams <- lapply(object@DataList[useImps], function(d) {
-      parTable(lavaan(model = PT, data = d, slotOptions = lavoptions,
-                      group = group, cluster = cluster))$est
-    })
+    #FIXME: below is legacy code, no longer needed?
+    # fit <- lavaan(parTable(object), data = dataList[[ useImps[1] ]],
+    #               slotOptions = lavoptions, group = group, cluster = cluster)
+    # ## use saturated parameter table as new model
+    # PT <- lavaan::lav_partable_unrestricted(fit)
+    # ## fit saturated parameter table to each imputation, return estimates
+    # satParams <- lapply(object@DataList[useImps], function(d) {
+    #   parTable(lavaan(model = PT, data = d, slotOptions = lavoptions,
+    #                   group = group, cluster = cluster))$est
+    # })
+    # ## fix them to pooled estimates
+    # PT$ustart <- colMeans(do.call(rbind, satParams))
+
+    PT <- object@h1List[[ useImps[1] ]]$PT
+    coefList <- lapply(object@h1List[useImps], function(x) x$PT$ustart)
+    PT$ustart <- colMeans(do.call(rbind, coefList))
+
     ## set all parameters fixed
     PT$free <- 0L
     PT$user <- 1L
-    ## fix them to pooled estimates
-    PT$ustart <- colMeans(do.call(rbind, satParams))
     PT$start <- NULL
     PT$est <- NULL
     PT$se <- NULL
@@ -396,7 +477,8 @@ getLLs <- function(object, useImps, saturated = FALSE) {
     PT$free <- 0L
     PT$user <- 1L
     ## fix them to pooled estimates
-    fixedValues <- getMethod("coef","lavaan.mi")(object, type = "user")
+    fixedValues <- getMethod("coef","lavaan.mi")(object, type = "user",
+                                                 omit.imps = omit.imps)
     PT$ustart <- fixedValues
     PT$start <- NULL
     PT$est <- NULL
@@ -406,44 +488,39 @@ getLLs <- function(object, useImps, saturated = FALSE) {
     PT <- PT[params, ]
   }
   ## return log-likelihoods
-  sapply(object@DataList[useImps], function(d) {
+  sapply(dataList, function(d) {
     lavaan::logLik(lavaan(PT, data = d, slotOptions = lavoptions,
                           group = group, cluster = cluster))
   })
+  #TODO: use "dry-run" trick from blavaan:::postpred() to save computing time
 }
 
 ##' @importFrom stats pf pchisq
 ##' @importFrom lavaan lavListInspect parTable
-D3.LRT <- function(object, h1 = NULL, asymptotic = FALSE) {
+D3.LRT <- function(object, h1 = NULL, useImps, asymptotic = FALSE,
+                   omit.imps = c("no.conv","no.se")) {
+  ## NOTE: Need to pass omit.imps= to getLLs(), which calls coef() method
+
   N <- lavListInspect(object, "ntotal")
-  useImps <- sapply(object@convergence, "[[", i = "converged")
-  # m <- length(object@testList)
-  nImps <- sum(useImps)
-  if (!is.null(h1)) {
-    if (any(useImps != sapply(h1@convergence, "[[", i = "converged")))
-      warning('The models being compared did not converge on the same set of ',
-              'imputations. Likelihood ratio test conducted using only the ',
-              'imputations for which both models converged.')
-    useImps <- useImps & sapply(h1@convergence, "[[", i = "converged")
-    nImps <- sum(useImps)
-  }
+  m <- length(useImps)
+
   if (is.null(h1)) {
-    DF <- object@testList[[ which(useImps)[1] ]][[1]][["df"]]
+    DF <- object@testList[[ useImps[1] ]]$standard[["df"]]
   } else {
-    DF1 <- h1@testList[[ which(useImps)[1] ]][[1]][["df"]]
-    DF0 <- object@testList[[ which(useImps)[1] ]][[1]][["df"]]
+    DF1 <- h1@testList[[ useImps[1] ]]$standard[["df"]]
+    DF0 <- object@testList[[ useImps[1] ]]$standard[["df"]]
     DF <- DF0 - DF1
     if (DF < 0) stop('The "object" model must be nested within (i.e., have ',
                      'fewer degrees of freedom than) the "h1" model.')
   }
 
   ## calculate m log-likelihoods under pooled H0 estimates
-  LL0 <- getLLs(object, useImps)
+  LL0 <- getLLs(object, useImps, omit.imps = omit.imps)
   ## calculate m log-likelihoods under pooled H1 estimates
   if (is.null(h1)) {
-    LL1 <- getLLs(object, useImps, saturated = TRUE)
+    LL1 <- getLLs(object, useImps, saturated = TRUE, omit.imps = omit.imps)
   } else {
-    LL1 <- getLLs(h1, useImps)
+    LL1 <- getLLs(h1, useImps, omit.imps = omit.imps)
   }
   #FIXME: check whether LL1 or LL0 returned errors?  add try()?
 
@@ -451,14 +528,14 @@ D3.LRT <- function(object, h1 = NULL, asymptotic = FALSE) {
   LRT_con <- mean(-2*(LL0 - LL1)) # getLLs() already applies [useImps]
   ## average chisq across imputations
   if (is.null(h1)) {
-    LRT_bar <- mean(sapply(object@testList[useImps], function(x) x[[1]]$stat))
+    LRT_bar <- mean(sapply(object@testList[useImps], function(x) x$standard$stat))
   } else {
-    LRT_bar <- mean(sapply(object@testList[useImps], function(x) x[[1]]$stat) -
-                      sapply(h1@testList[useImps], function(x) x[[1]]$stat))
+    LRT_bar <- mean(sapply(object@testList[useImps], function(x) x$standard$stat) -
+                      sapply(h1@testList[useImps], function(x) x$standard$stat))
   }
   ## calculate average relative increase in variance
-  a <- DF*(nImps - 1)
-  ariv <- ((nImps + 1) / a) * (LRT_bar - LRT_con)
+  a <- DF*(m - 1)
+  ariv <- ((m + 1) / a) * (LRT_bar - LRT_con)
   test.stat <- LRT_con / (DF*(1 + ariv))
   if (is.na(test.stat)) stop('D3 test statistic could not be calculated. ',
                              'Try the D2 pooling method.') #FIXME: check whether model-implied Sigma is NPD
@@ -468,7 +545,8 @@ D3.LRT <- function(object, h1 = NULL, asymptotic = FALSE) {
   }
   if (asymptotic) {
     out <- c("chisq" = test.stat * DF, df = DF,
-             pvalue = pchisq(test.stat * DF, df = DF, lower.tail = FALSE))
+             pvalue = pchisq(test.stat * DF, df = DF, lower.tail = FALSE),
+             ariv = ariv, fmi = ariv / (1 + ariv))
   } else {
     ## F statistic
     if (a > 4) {
@@ -478,7 +556,8 @@ D3.LRT <- function(object, h1 = NULL, asymptotic = FALSE) {
       # v4 <- (DF + 1)*(m - 1)*(1 + (1 / ariv))^2 / 2 # Grund et al. (eq. 9)
     }
     out <- c("F" = test.stat, df1 = DF, df2 = v4,
-             pvalue = pf(test.stat, df1 = DF, df2 = v4, lower.tail = FALSE))
+             pvalue = pf(test.stat, df1 = DF, df2 = v4, lower.tail = FALSE),
+             ariv = ariv, fmi = ariv / (1 + ariv))
   }
   ## add log-likelihood and AIC/BIC for target model
   if (is.null(h1)) {
@@ -499,16 +578,32 @@ D3.LRT <- function(object, h1 = NULL, asymptotic = FALSE) {
 
 ##' @importFrom stats pchisq
 ##' @importFrom lavaan lavListInspect
-robustify <- function(ChiSq, object, h1 = NULL, useImps) {
-  scaleshift <- lavListInspect(object, "options")$test == "scaled.shifted"
+robustify <- function(ChiSq, object, h1 = NULL, baseline = FALSE, useImps) {
+  test.names <- lavListInspect(object, "options")$test
+  # lavaan 0.6-5: for now, we only acknowledge the first non-standard @test
+  if (length(test.names) > 1L) {
+    ## remove standard and any bootstrapped tests
+    rm.idx <- which(test.names %in% c("standard","bootstrap","bollen.stine"))
+    if (length(rm.idx) > 0L) {
+      test.names <- test.names[-rm.idx]
+    }
+    ## only acknowledge the first scaled test statistic
+    if (length(test.names) > 1L) {
+      test.names <- test.names[1]
+    }
+  }
+  scaleshift <- any(test.names %in% "scaled.shifted")
 
-  d0 <- mean(sapply(object@testList[useImps], function(x) x[[2]][["df"]]))
-  c0 <- mean(sapply(object@testList[useImps],
-                    function(x) x[[2]][["scaling.factor"]]))
+  if (baseline) {
+    TEST <- lapply(object@baselineList[useImps], "[[", i = "test")
+  } else TEST <- object@testList[useImps]
+
+  d0 <- mean(sapply(TEST, function(x) x[[ test.names[1] ]][["df"]]))
+  c0 <- mean(sapply(TEST, function(x) x[[ test.names[1] ]][["scaling.factor"]]))
   if (!is.null(h1)) {
-    d1 <- mean(sapply(h1@testList[useImps], function(x) x[[2]][["df"]]))
+    d1 <- mean(sapply(h1@testList[useImps], function(x) x[[ test.names[1] ]][["df"]]))
     c1 <- mean(sapply(h1@testList[useImps],
-                      function(x) x[[2]][["scaling.factor"]]))
+                      function(x) x[[ test.names[1] ]][["scaling.factor"]]))
     delta_c <- (d0*c0 - d1*c1) / (d0 - d1)
     ChiSq["chisq.scaled"] <- ChiSq[["chisq"]] / delta_c
     ChiSq["df.scaled"] <- d0 - d1
@@ -521,8 +616,7 @@ robustify <- function(ChiSq, object, h1 = NULL, useImps) {
     ChiSq["df.scaled"] <- d0
     if (scaleshift) {
       ## add average shift parameter (or average of sums, if nG > 1)
-      shift <- mean(sapply(object@testList[useImps],
-                           function(x) sum(x[[2]][["shift.parameter"]]) ))
+      shift <- mean(sapply(TEST, function(x) sum(x[[ test.names[1] ]][["shift.parameter"]]) ))
       ChiSq["chisq.scaled"] <- ChiSq[["chisq.scaled"]] + shift
       ChiSq["pvalue.scaled"] <- pchisq(ChiSq[["chisq.scaled"]],
                                        df = ChiSq[["df.scaled"]],
